xquery version "3.1";

(:~
 : This module provides dictionary annotations for Ahiqar based on 
 : Sedra, an online dictionary for the Syriac language.
 :
 : @author Mathias Göbel
 : @version 0.0.0
 : @since 12.2.0
 : @see https://subugoe.pages.gwdg.de/ahiqar/api-documentation/
 : @see https://subugoe.pages.gwdg.de/ahiqar/api-documentation/page/text-api-specs/
 : :)

module namespace sedra="http://ahiqar.uni-goettingen.de/ns/annotations/sedra";

declare namespace tei="http://www.tei-c.org/ns/1.0";

import module namespace commons="http://ahiqar.uni-goettingen.de/ns/commons" at "../commons.xqm";
import module namespace hc="http://expath.org/ns/http-client" at "java:org.expath.exist.HttpClientModule";


declare function sedra:main(
    $pages as element(tei:TEI)+,
    $teixml-uri as xs:string,
    $page as xs:string) {
    
    for $token in $pages//tei:w
    let $id := string($token/@xml:id)
    return
        map {
            "id": $commons:anno-ns || "/" || $teixml-uri || "/annotation-sedra/" || $id,
            "type": "Annotation",
            "body":
                map {
                    "type": "TextualBody",
                    "value": sedra:get-value(string($token)),
                    "format": "text/plain",
                    "x-content-type": "dictionary"
                },
            "target": array { $id }
        }
};

declare function sedra:get-value($token as xs:string) {
    let $url := "https://api.dev.ahiqar.sub.uni-goettingen.de/sedra-service/syriac/" || string($token)
    let $sedra :=
        hc:send-request(
            <hc:request method="GET" href="{$url}" />
            )[2] => util:binary-to-string() => parse-json()
    return
        for $i in $sedra?*
        return
            (
                map:for-each($i, function($k, $v){
                    if( not( $k = ("gender", "tense", "category", "stem", "number", "person", "syriac") ) ) then () else
                    $k || ": " || string($v) || "; ",
                    if($k eq "glosses") then $k || ": " || string-join(distinct-values($v?*), "; ") else ()
                    })
                )
            => string-join("; ")
};